package com.qs.mp.channel.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.eptok.yspay.opensdkjava.fund.MercFundApi;
import com.eptok.yspay.opensdkjava.pojo.vo.OnlineReqDataVo;
import com.eptok.yspay.opensdkjava.util.DateUtil;
import com.qs.mp.channel.domain.Channel;
import com.qs.mp.channel.domain.ChannelBankCard;
import com.qs.mp.channel.domain.ChannelMoneyLog;
import com.qs.mp.channel.domain.ChannelWithdraw;
import com.qs.mp.channel.domain.vo.ChannelWithdrawVO;
import com.qs.mp.channel.mapper.ChannelWithdrawMapper;
import com.qs.mp.channel.service.IChannelBankCardService;
import com.qs.mp.channel.service.IChannelMoneyLogService;
import com.qs.mp.channel.service.IChannelService;
import com.qs.mp.channel.service.IChannelWithdrawService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qs.mp.common.constant.YsPayMethodConstants;
import com.qs.mp.common.constant.YsServerApiConstants;
import com.qs.mp.common.domain.YsCallLog;
import com.qs.mp.common.enums.ChannelMoneyEnum;
import com.qs.mp.common.enums.ChannelWithdrawStatusEnum;
import com.qs.mp.common.enums.YsCallBizTypeEnum;
import com.qs.mp.common.exception.ServiceException;
import com.qs.mp.common.service.IYsCallLogService;
import com.qs.mp.common.utils.LogUtil;
import com.qs.mp.common.utils.WebhookService;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

/**
 * <p>
 * 渠道提现表 服务实现类
 * </p>
 *
 * @author quanshu
 * @since 2022-03-04
 */
@Service
public class ChannelWithdrawServiceImpl extends ServiceImpl<ChannelWithdrawMapper, ChannelWithdraw> implements IChannelWithdrawService {

  private final Logger logger = LoggerFactory.getLogger(this.getClass().getSimpleName());


  @Value("${ys-pay.private-key-path}")
  private String privateKeyPath;

  @Value("${ys-pay.public-key-path}")
  private String publicKeyPath;


  @Value("${ys-pay.partner-id}")
  private String partnerId;

  @Value("${ys-pay.private-key-pass}")
  private String privateKeyPass;

  @Value("${ys-pay.withdraw-notify-url}")
  private String ysWithdrawNotifyUrl;

  @Value(value = "${server.env}")
  private String env;

  @Autowired
  private IYsCallLogService ysCallLogService;

  @Autowired
  private IChannelBankCardService channelBankCardService;

  @Autowired
  private IChannelService channelService;

  @Autowired
  private IChannelMoneyLogService channelMoneyLogService;

  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean apply(Channel channel, ChannelWithdraw channelWithdraw) {

    // 更新用户的银行卡信息
    ChannelBankCard bankCard = channelBankCardService.getOne(new LambdaQueryWrapper<ChannelBankCard>()
        .eq(ChannelBankCard::getChannelId, channel.getChannelId()));
    if (null == bankCard) {
//      throw new RuntimeException("请先绑定银行卡");
//    }
      bankCard = new ChannelBankCard();
      bankCard.setChannelId(channel.getChannelId());
      bankCard.setCardNo(channelWithdraw.getCardNo());
      bankCard.setBankName(channelWithdraw.getBankName());
      bankCard.setBranchName(channelWithdraw.getBranchName());
      bankCard.setUserName(channelWithdraw.getUserName());
      channelBankCardService.save(bankCard);
    } else {
      bankCard.setCardNo(channelWithdraw.getCardNo());
      bankCard.setBankName(channelWithdraw.getBankName());
      bankCard.setBranchName(channelWithdraw.getBranchName());
      bankCard.setUserName(channelWithdraw.getUserName());
      channelBankCardService.updateById(bankCard);
    }
    boolean rst = save(channelWithdraw);

    ChannelMoneyLog moneyLog = new ChannelMoneyLog();
    moneyLog.setChannelId(channel.getChannelId());
    moneyLog.setType(ChannelMoneyEnum.WITHDRAW);
    moneyLog.setLogMoney(channelWithdraw.getMoney());
    moneyLog.setBizTime(new Date());
    moneyLog.setRefId(String.valueOf(channelWithdraw.getId()));
    moneyLog.setLogText("银行卡尾号 " + channelWithdraw.getCardNo().substring(channelWithdraw.getCardNo().length() - 4));
    channelMoneyLogService.changeMoney(moneyLog);

    if (channelWithdraw.getChargeAmt() > 0) {
      // 手续费
      ChannelMoneyLog feeMoneyLog = new ChannelMoneyLog();
      feeMoneyLog.setChannelId(channel.getChannelId());
      feeMoneyLog.setType(ChannelMoneyEnum.WITHDRAW_FEE);
      feeMoneyLog.setLogMoney(channelWithdraw.getChargeAmt());
      feeMoneyLog.setBizTime(new Date());
      feeMoneyLog.setRefId(String.valueOf(channelWithdraw.getId()));
      channelMoneyLogService.changeMoney(feeMoneyLog);
    }
    StringBuilder sb = new StringBuilder();
    sb.append("提现申请\n");
    sb.append("申请人：").append(channel.getName()).append("\n");
    sb.append("提现金额：").append(new BigDecimal(channelWithdraw.getMoney()).divide(new BigDecimal(100), 2, RoundingMode.DOWN)).append("元\n");
    WebhookService.sendBizData(sb.toString());
    // 创建提现记录
    return rst;
  }

  @Override
  public List<ChannelWithdrawVO> listWithdrawVO(Wrapper<ChannelWithdraw> queryWrapper) {
    return getBaseMapper().listWithdrawVO(queryWrapper);
  }

  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean verify(ChannelWithdraw withdraw, ChannelWithdrawStatusEnum status, String verifyContent) {
    if (status == ChannelWithdrawStatusEnum.FINISHED) {
      boolean rtn = update(new LambdaUpdateWrapper<ChannelWithdraw>()
          .set(ChannelWithdraw::getStatus, status)
          .set(ChannelWithdraw::getTransferTime, new Date())
          .eq(ChannelWithdraw::getId, withdraw.getId())
          .eq(ChannelWithdraw::getStatus, ChannelWithdrawStatusEnum.WITHDRAWING));
      Assert.isTrue(rtn, "提现申请审核通过，更新提现申请状态失败。id:" + withdraw.getId());

      // 更新冻结金额
      Channel channel = channelService.getById(withdraw.getChannelId());
      Assert.isTrue(channel.getFrozenMoney() >= withdraw.getMoney() + withdraw.getChargeAmt(),
          "提现申请审核通过，更新冻结金额时冻结金额为:" + channel.getFrozenMoney() + ",本次要释放冻结金额为：" + withdraw.getMoney() + withdraw.getChargeAmt());

      boolean updateRtn = channelService.update(new LambdaUpdateWrapper<Channel>()
          .set(Channel::getFrozenMoney, channel.getFrozenMoney() - withdraw.getMoney() - withdraw.getChargeAmt())
          .eq(Channel::getChannelId, withdraw.getChannelId())
          .eq(Channel::getFrozenMoney, channel.getFrozenMoney()));
      Assert.isTrue(updateRtn, "提现申请审核通过，更新账号冻结金额。channelId:" + channel.getChannelId());
    }
    // 银盛提现接口调用
    /*
    else if (status == ChannelWithdrawStatusEnum.WITHDRAW_PROCESS) {
      String tradeNo = DateUtil.getDateNowYmd() + DateUtil.getRandom(14);;

      // 更新状态为提现处理中
      boolean rtn = update(new LambdaUpdateWrapper<ChannelWithdraw>()
          .set(ChannelWithdraw::getTradeNo, tradeNo)
          .set(ChannelWithdraw::getStatus, status)
          .eq(ChannelWithdraw::getId, withdraw.getId())
          .in(ChannelWithdraw::getStatus, ChannelWithdrawStatusEnum.WITHDRAWING, ChannelWithdrawStatusEnum.YS_WITHDRAW_FAILED));
      Assert.isTrue(rtn, "调用银盛提现接口失败。id:" + withdraw.getId());

      // 调用银盛发起提现
      OnlineReqDataVo req = new OnlineReqDataVo();

      //银盛支付服务器主动通知商户网站里指定的页面http路径
      req.setNotifyUrl(ysWithdrawNotifyUrl);

      //商户在银盛支付平台开设的用户号[商户号],接入时需要替换成自己的
      req.setPartnerId("hyfz_test2");
      req.setReqUrl(YsServerApiConstants.COMMON_API);
      req.setPrivateKeyFilePath(privateKeyPath);
      req.setPrivateKeyPassword(privateKeyPass);
      req.setYsPublicKeyFilePath(publicKeyPath);
      //实时提现（一般户到银行卡）业务参数
      Map<String, Object> bizContent = new HashMap<>();
      //商户生成的订单号，生成规则前8位必须为交易日期，如20180525，范围跨度支持包含当天在内的前后一天
      bizContent.put("out_trade_no", tradeNo);
      //商户号
      bizContent.put("merchant_usercode", partnerId);
      //暂时只支持币种：CNY（人民币）
      bizContent.put("currency", "CNY");
      //提现的总金额。单位为：RMB Yuan。取值范围为[0.01，99999999.99]，精确到小数点后两位。Number(10,2)指10位长度，2位精度
      if ("prod".equals(env)) {
        bizContent.put("total_amount", new BigDecimal(withdraw.getMoney()).divide(new BigDecimal(100), 2, RoundingMode.HALF_UP).toString());
      } else {
        bizContent.put("total_amount", "1.00");
      }
      //订单说明
      bizContent.put("subject", "提现");
      //商户日期(该参数做交易与查询时需要一致) 该日期需在当日的前后一天时间范围之内
      bizContent.put("shopdate", DateUtil.getDateNowYmd());
      //银行帐号
      bizContent.put("bank_account_no", withdraw.getCardNo());
      req.setParamData(bizContent);
      String result = null;

      YsCallLog ysCallLog = new YsCallLog();
      try{
        logger.info("实时提现（一般户到银行卡）调用sdk接口addScanMerc请求入参为:"+ JSONObject.toJSONString(req));
        //根据返回结果处理自己的业务逻辑,result内容详见接口文档
        result = MercFundApi.withdrawQuick(req);
        ysCallLog.setResJson(result);
        logger.info("实时提现（一般户到银行卡）调用sdk接口addScanMerc请求出参为:"+ result);
      }catch (Exception e){
        logger.info("实时提现（一般户到银行卡）失败:"+ e.getMessage());
        throw new ServiceException("调用银盛发起提现失败");
      }

      ysCallLog.setBizId(String.valueOf(withdraw.getId()));
      ysCallLog.setBizType(YsCallBizTypeEnum.WITHDRAW);
      ysCallLog.setInterfaceId(YsPayMethodConstants.GENERAL_ACCOUNT_WITHDRAW_METHOD);
      ysCallLog.setReqJson(JSONObject.toJSONString(bizContent));
      ysCallLogService.save(ysCallLog);
      */

     else if (status == ChannelWithdrawStatusEnum.FAILED) {
      boolean rtn = update(new LambdaUpdateWrapper<ChannelWithdraw>()
          .set(ChannelWithdraw::getStatus, status)
          .set(ChannelWithdraw::getVerifyContent, verifyContent)
          .eq(ChannelWithdraw::getId, withdraw.getId())
          .eq(ChannelWithdraw::getStatus, ChannelWithdrawStatusEnum.WITHDRAWING));
      Assert.isTrue(rtn, "提现申请审核不通过，更新提现申请状态失败。id:" + withdraw.getId());

      // 退回冻结金额
      Channel channel = channelService.getById(withdraw.getChannelId());
      Assert.isTrue(channel.getFrozenMoney() >= withdraw.getMoney() + withdraw.getChargeAmt(),
          "提现申请审核不通过，解冻冻结金额时冻结金额为:" + channel.getFrozenMoney() + ",本次要解冻金额为：" + withdraw.getMoney() + withdraw.getChargeAmt());
      ChannelMoneyLog moneyLog = new ChannelMoneyLog();
      moneyLog.setChannelId(channel.getChannelId());
      moneyLog.setType(ChannelMoneyEnum.WITHDRAW_BACK);
      moneyLog.setLogMoney(withdraw.getMoney() + withdraw.getChargeAmt());
      moneyLog.setBizTime(new Date());
      moneyLog.setRefId(String.valueOf(withdraw.getId()));
      moneyLog.setLogText(null);
      channelMoneyLogService.changeMoney(moneyLog);
    }
    return true;
  }
}
