package com.qs.mp.pay.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.request.AlipayTradeCreateRequest;
import com.alipay.api.response.AlipayTradeCreateResponse;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.eptok.yspay.opensdkjava.common.Constants;
import com.eptok.yspay.opensdkjava.fund.MercFundApi;
import com.eptok.yspay.opensdkjava.orderpay.AppPayApi;
import com.eptok.yspay.opensdkjava.pojo.vo.OnlineReqDataVo;
import com.eptok.yspay.opensdkjava.util.DateUtil;
import com.eptok.yspay.opensdkjava.util.HttpClientUtil;
import com.eptok.yspay.opensdkjava.util.StringUtil;
import com.eptok.yspay.opensdkjava.util.YsOnlineSignUtils;
import com.qs.mp.channel.domain.ChannelWithdraw;
import com.qs.mp.channel.service.IChannelGoodsOrderService;
import com.qs.mp.channel.service.IChannelOrderService;
import com.qs.mp.channel.service.IChannelWithdrawService;
import com.qs.mp.common.constant.YsPayMethodConstants;
import com.qs.mp.common.constant.YsServerApiConstants;
import com.qs.mp.common.domain.YsCallLog;
import com.qs.mp.common.enums.AppSourceEnum;
import com.qs.mp.common.enums.AsyncTaskTypeEnum;
import com.qs.mp.common.enums.BizTypeEnum;
import com.qs.mp.common.enums.ChannelWithdrawStatusEnum;
import com.qs.mp.common.enums.MqTopicType;
import com.qs.mp.common.enums.PayOrderStatusEnum;
import com.qs.mp.common.enums.YsCallBizTypeEnum;
import com.qs.mp.common.exception.ServiceException;
import com.qs.mp.common.pulsar.PulsarClientService;
import com.qs.mp.common.service.IYsCallLogService;
import com.qs.mp.common.utils.DateUtils;
import com.qs.mp.common.utils.LogUtil;
import com.qs.mp.common.utils.StringUtils;
import com.qs.mp.common.utils.http.OkHttpUtil;
import com.qs.mp.framework.domain.AsyncTask;
import com.qs.mp.framework.service.IAsyncTaskService;
import com.qs.mp.pay.domain.PayOrder;
import com.qs.mp.pay.service.IPayOrderService;
import com.qs.mp.pay.service.IWalletService;
import com.qs.mp.system.service.id.BizIdGenerator;
import com.qs.mp.user.domain.UserTicketOrder;
import com.qs.mp.user.service.IUserDeliverOrderService;
import com.qs.mp.user.service.IUserTicketOrderService;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import org.apache.pulsar.client.api.PulsarClientException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import java.util.Objects;
import org.springframework.transaction.annotation.Transactional;

/**
 * @auther duota
 * @create 2021 2021/9/4 1:59 下午
 * @describe
 */
@Component
public class WalletServiceImpl implements IWalletService {

  protected final Logger logger = LoggerFactory.getLogger(this.getClass().getSimpleName());

  @Autowired
  private IPayOrderService payOrderService;

  @Autowired
  private IChannelOrderService channelOrderService;

  @Autowired
  private IUserTicketOrderService userTicketOrderService;

  @Autowired
  private IUserDeliverOrderService userDeliverOrderService;

  @Autowired
  private PulsarClientService pulsarClientService;

  @Autowired
  private BizIdGenerator bizIdGenerator;

  @Autowired
  private IAsyncTaskService asyncTaskService;

  @Autowired
  private IChannelWithdrawService channelWithdrawService;

  @Autowired
  private IYsCallLogService ysCallLogService;

  @Autowired
  private IChannelGoodsOrderService  channelGoodsOrderService;

  @Value("${pay.notifyUrl}")
  private String notifyUrl;  //支付成功前端跳转地址
  @Value("${pay.callbackUrl}")
  private String callbackUrl;  //支付回调地址
  @Value("${pay.baseUrl}")
  private String baseUrl;
  @Value("${wx-channel.appId}")
  private String channelAppId;
  @Value("${wx-user.appId}")
  private String userAppId;

  @Value("${pay.user-shopNo}")
  private String userPayShopNo;  //用户端商户号
  @Value("${pay.user-sign}")
  private String userPaySign;  //用户端商户签名

  @Value("${pay.msdq-shopNo}")
  private String msdqPayShopNo;

  @Value("${pay.msdq-shopNo}")
  private String msdqPaySign;

  @Value("${pay.channel-shopNo}")
  private String channelPayShopNo;  //渠道端商户号
  @Value("${pay.channel-sign}")
  private String channelPaySign;  //渠道端商户签名


  @Value("${ali-miniApp.appId}")
  private String aliMiniAppAppId;

  private static final String REQ_STATUS_SUCCESS = "R0001";
  private static final String PAY_RESOURCE = "2"; //2:微信支付
  private static final String PAY_RESOURCE_TYPE = "1017"; //2:微信小程序

  // 支付宝直连
  @Value("${ali-miniApp.appId}")
  private String aliAppId;

  @Value("${ali-miniApp.public-key}")
  private String aliPublicKey;

  @Value("${ali-miniApp.private-key}")
  private String aliPrivateKey;

  @Value("${ali-miniApp.serverUrl}")
  private String aliServerUrl;

  @Value("${alipay.callbackUrl}")
  private String alipayCallbackUrl;  //支付宝直连回调地址

  private AlipayClient alipayClient = null;


  @Value("${ys-pay.partner-id}")
  private String partnerId;

  @Value("${ys-pay.private-key-pass}")
  private String privateKeyPass;

  @Value("${ys-pay.private-key-path}")
  private String privateKeyPath;

  @Value("${ys-pay.public-key-path}")
  private String publicKeyPath;

  @Value("${ys-pay.master-seller-id}")
  private String masterSellerId;

  @Value("${ys-pay.child-seller-id}")
  private String childSellerId;

  @Value("${ys-pay.return-url}")
  private String ysReturnUrl;
  @Value("${ys-pay.pay-notify-url}")
  private String ysPayNotifyUrl;

  @Value("${ys-pay.withdraw-notify-url}")
  private String ysWithdrawNotifyUrl;

  @Override
  public JSONObject h5Pay(BizTypeEnum bizType, String bizId, int money, String orderRemark) {
    Map<String, String> params = new HashMap<>();
    String orderId = String.valueOf(bizIdGenerator.newId());
    String tradeNo = DateUtil.getDateNowYmd() + DateUtil.getRandom(14);
    // 返回页面拼接

    String returnUrl = ysReturnUrl;
    if (BizTypeEnum.TICKET_ORDER.equals(bizType)) {
      UserTicketOrder userTicketOrder = userTicketOrderService.getById(bizId);
      returnUrl = returnUrl + "?boxId=" + userTicketOrder.getBoxId() + "&orderId=" + bizId + "&istry=0";
    }
    params.put("method", YsPayMethodConstants.H5_PAY_METHOD);
    params.put("partner_id", partnerId);
    params.put("timestamp", DateUtil.getDateNow());
    params.put("charset", Constants.CHARSET_UTF_8);
    params.put("sign_type", "RSA");
    params.put("notify_url", ysPayNotifyUrl);
    params.put("return_url", returnUrl);
    params.put("seller_id", partnerId);
    params.put("version","3.0");
    params.put("out_trade_no", tradeNo);
    params.put("shopdate", DateUtil.getDateNowYmd());
    params.put("subject", orderRemark);
    params.put("total_amount", new BigDecimal(money).divide(new BigDecimal(100), 2, RoundingMode.HALF_UP).toString());
    params.put("timeout_express","96h");
    params.put("business_code", "3010002");
    try {
      String sign = YsOnlineSignUtils.sign(params, privateKeyPass, privateKeyPath);
      params.put("sign", sign);
    } catch (Exception e) {
      logger.error("签名失败,bizId=" + bizId, e);
      throw new ServiceException("支付发起签名失败");
    }

//    String result = OkHttpUtil.post("https://openapi.ysepay.com/gateway.do", params);
//
    // 保存调用记录
    YsCallLog ysCallLog = new YsCallLog();
    ysCallLog.setBizId(orderId);
    ysCallLog.setBizType(YsCallBizTypeEnum.ORDER_PAY);
    ysCallLog.setInterfaceId(YsPayMethodConstants.H5_PAY_METHOD);
    ysCallLog.setReqJson(JSONObject.toJSONString(params));
    ysCallLogService.save(ysCallLog);

//
//    String result = "https://openapi.ysepay.com/gateway.do?";
//    for (String key : params.keySet()) {
//      result += key + "=" + params.get(key);
//      // 如果是最后一个,则不再末尾加上&
//      if (key != params.keySet().toArray()[params.keySet().size() - 1]) {
//          result += "&";
//      }
//    }
//    System.out.println("result = " + result);

//
//    //保存订单记录
    PayOrder payOrder = new PayOrder();
    payOrder.setOrderId(orderId);
    payOrder.setBizType(bizType);
    payOrder.setBizId(bizId);
    payOrder.setTradeNo(tradeNo);
    payOrder.setOrderTime(DateUtils.getTime());
    payOrder.setPayResource("3"); // 支付宝直连
    payOrder.setTransTypeCode("1020"); // 支付宝小程序
    payOrder.setTransactionAmount(money);
    payOrder.setOrderRemark(orderRemark);
    payOrder.setOrderName(orderRemark);
    payOrder.setOrderStatus(PayOrderStatusEnum.WAIT.getValue());
    boolean ret = payOrderService.save(payOrder);
    if (!ret) {
      LogUtil.error(logger, "支付订单数据保存失败.");
      throw new ServiceException("支付订单保存失败");
    }
//
//    JSONObject data = new JSONObject();
//    data.put("payUrl", result);


    JSONObject data = JSONObject.parseObject(JSONObject.toJSONString(params));
    return data;
  }


  @Override
  public JSONObject directAliPay(BizTypeEnum bizType, String bizId, String openId, int money, String orderRemark) {
    if (null == alipayClient) {
      alipayClient = new DefaultAlipayClient(aliServerUrl, aliAppId, aliPrivateKey, "json", "UTF-8", aliPublicKey, "RSA2");
    }
    Map<String, Object> params = new HashMap<String, Object>();
    String orderId = String.valueOf(bizIdGenerator.newId());
    params.put("out_trade_no", orderId);
    params.put("buyer_id", openId);
    params.put("subject", orderRemark);
    params.put("total_amount", new BigDecimal(money).divide(new BigDecimal(100), 2, RoundingMode.HALF_UP));
    Object obj = JSONObject.toJSON(params);
    AlipayTradeCreateRequest alipayRequest = new AlipayTradeCreateRequest();
    alipayRequest.setNotifyUrl(alipayCallbackUrl);
    alipayRequest.setBizContent(obj.toString());
    AlipayTradeCreateResponse alipayResponse = null;
    try {
      alipayResponse = alipayClient.execute(alipayRequest);
      if (null == alipayResponse || !alipayResponse.isSuccess()) {
        LogUtil.error(logger, "创建直连支付宝交易失败。request:{0}, response:{1}", new Object[]{JSONObject.toJSON(alipayRequest),
            alipayResponse});
        throw new ServiceException("订单创建失败");
      }



    }catch (AlipayApiException e) {
      if(e.getCause() instanceof java.security.spec.InvalidKeySpecException){
        logger.error("商户私钥格式不正确，请确认application.properties文件中是否配置正确");
      }
      logger.error("创建支付宝交易异常。request:" + JSON.toJSONString(alipayRequest), e);
      throw new ServiceException("订单创建失败");
    }

    String trade_no = alipayResponse.getTradeNo();
    //保存订单记录
    PayOrder payOrder = new PayOrder();
    payOrder.setOrderId(orderId);
    payOrder.setBizType(bizType);
    payOrder.setBizId(bizId);
    payOrder.setOrderNo(trade_no);
    payOrder.setOrderTime(DateUtils.getTime());
    payOrder.setPayResource("10"); // 支付宝直连
    payOrder.setTransTypeCode("1019"); // 支付宝小程序
    payOrder.setTransactionAmount(money);
    payOrder.setOrderRemark(orderRemark);
    payOrder.setOrderName(orderRemark);
    payOrder.setOrderStatus(PayOrderStatusEnum.WAIT.getValue());
    boolean ret = payOrderService.save(payOrder);
    if (!ret) {
      LogUtil.error(logger, "支付订单数据保存失败.");
      throw new ServiceException("支付订单保存失败");
    }

    JSONObject data = new JSONObject();
    data.put("pay_info", trade_no);
    return data;
  }

  @Override
  public JSONObject aliPay(BizTypeEnum bizType, String bizId, String openId, int money, String orderRemark) {
    String reqUrl = baseUrl + "/pay/payMoneyCTB";
    String appId = aliMiniAppAppId;
    String payShopNo = userPayShopNo;
    String paySign = userPaySign;
    JSONObject params = buildPayOrderReqData(payShopNo, paySign, money, openId, appId, orderRemark, "1", "1014");
    String result = OkHttpUtil.postJsonParams(reqUrl, params.toJSONString());
    logger.info("request params:" + params.toJSONString() + "  result:" + result);
    if (StringUtils.isBlank(result)) {
      LogUtil.error(logger, "创建支付订单接口失败");
      throw new ServiceException("创建三方支付订单失败");
    }
    JSONObject resultJson = JSONObject.parseObject(result);
    String code = resultJson.getString("code");
    String orderNo = resultJson.getString("order_no");
    String payInfo = resultJson.getString("pay_info");
    String orderstatus = resultJson.getString("orderstatus");
    String resultMsg = resultJson.getString("result");
    if (!REQ_STATUS_SUCCESS.equalsIgnoreCase(code)) {
      throw new ServiceException(resultMsg);
    }
    //保存订单记录
    PayOrder payOrder = buildPayOrder(params);
    payOrder.setOrderId(resultJson.getString("shop_order_no"));
    payOrder.setOrderNo(orderNo);
    payOrder.setCode(code);
    payOrder.setBizType(bizType);
    payOrder.setBizId(bizId);
    payOrder.setOrderStatus(orderstatus);
    boolean ret = payOrderService.save(payOrder);
    if (!ret) {
      LogUtil.error(logger, "支付订单数据库保存失败.");
      throw new ServiceException("支付订单保存失败");
    }
    JSONObject data = new JSONObject();
    data.put("pay_info", payInfo);
    return data;
  }

  @Override
  public JSONObject pay(BizTypeEnum bizType, String bizId, String openId, int money,
      String orderRemark, Integer appSource) {
    String reqUrl = baseUrl + "/pay/payMoneyCTB";
    String appId = channelAppId;
    String payShopNo = channelPayShopNo;
    String paySign = channelPaySign;

    AppSourceEnum appSourceEnum = AppSourceEnum.getByValue(appSource);
    if (bizType == BizTypeEnum.TICKET_ORDER || bizType == BizTypeEnum.DELIVER_ORDER) {
      appId = appSourceEnum.getAppId();
      // 区分微信小程序
      if (AppSourceEnum.MSDQ.equals(appSourceEnum)) {
        payShopNo = msdqPayShopNo;
        paySign = msdqPaySign;
      } else {
        payShopNo = userPayShopNo;
        paySign = userPaySign;
      }

    }
    JSONObject params = buildPayOrderReqData(payShopNo, paySign, money, openId, appId, orderRemark, PAY_RESOURCE, PAY_RESOURCE_TYPE);
    String result = OkHttpUtil.postJsonParams(reqUrl, params.toJSONString());
    logger.info("request params:" + params.toJSONString() + "  result:" + result);
    if (StringUtils.isBlank(result)) {
      LogUtil.error(logger, "创建支付订单接口失败");
      throw new ServiceException("创建三方支付订单失败");
    }
    JSONObject resultJson = JSONObject.parseObject(result);
    String code = resultJson.getString("code");
    String orderNo = resultJson.getString("order_no");
    JSONObject data = resultJson.getJSONObject("pay_info");
    String orderstatus = resultJson.getString("orderstatus");
    String resultMsg = resultJson.getString("result");
    if (!REQ_STATUS_SUCCESS.equalsIgnoreCase(code)) {
      throw new ServiceException(resultMsg);
    }
    //保存订单记录
    PayOrder payOrder = buildPayOrder(params);
    payOrder.setOrderId(resultJson.getString("shop_order_no"));
    payOrder.setOrderNo(orderNo);
    payOrder.setCode(code);
    payOrder.setBizType(bizType);
    payOrder.setBizId(bizId);
    payOrder.setOrderStatus(orderstatus);
    boolean ret = payOrderService.save(payOrder);
    if (!ret) {
      LogUtil.error(logger, "支付订单数据库保存失败.");
      throw new ServiceException("支付订单保存失败");
    }
    return data;

  }

  @Override
  public boolean refund(String bizId, Integer refundAmount, String remark) {
    // 获取支付成功的订单号
    PayOrder payOrder = payOrderService.getOne(new LambdaQueryWrapper<PayOrder>()
            .eq(PayOrder::getBizId, bizId)
            .eq(PayOrder::getOrderStatus, PayOrderStatusEnum.SUCCESS));
    if (Objects.isNull(payOrder)) {
      throw new ServiceException("支付订单不存在");
    }

    String reqUrl = baseUrl + "/pay/refundOrder";
    JSONObject sendData = new JSONObject();

    if (BizTypeEnum.CHANNEL_ORDER.equals(payOrder.getBizType())) {
      sendData.put("shop_no", channelPayShopNo);
      sendData.put("sign",channelPaySign);
    } else {
      sendData.put("shop_no", msdqPayShopNo);
      sendData.put("sign", msdqPaySign);
    }
    sendData.put("timestamp",System.currentTimeMillis());
    sendData.put("order_no",payOrder.getOrderId());
    sendData.put("refund_amount", refundAmount);
    sendData.put("order_remark", remark);
    sendData.put("type","sno");
    String result = OkHttpUtil.postJsonParams(reqUrl, sendData.toJSONString());
    JSONObject jsonObject = JSONObject.parseObject(result);
    String code = jsonObject.getString("code");
    logger.info("用户订单退款 -> request params:" + sendData.toJSONString() + "  result:" + result);
    // 退款成功,更新订单状态
    if ("R0001".equals(code)) {
      boolean ret = payOrderService.update(new LambdaUpdateWrapper<PayOrder>()
              .set(PayOrder::getOrderStatus, PayOrderStatusEnum.REFUND.getValue())
              .eq(PayOrder::getOrderId, payOrder.getOrderId())
              .eq(PayOrder::getOrderStatus, PayOrderStatusEnum.SUCCESS.getValue()));

      if (!ret) {
        LogUtil.error(logger, "订单退款状态数据库更新失败, BizId:" + bizId);
      }
      return true;
    }
    LogUtil.error(logger, "订单退款失败,code:" + code);
    throw new ServiceException("订单退款失败");
  }


  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean withdrawal(String bizId) {
    /*ChannelWithdraw channelWithdraw = channelWithdrawService.getById(bizId);

    if (!ChannelWithdrawStatusEnum.WITHDRAWING.equals(channelWithdraw.getStatus())) {
        throw new ServiceException("提现状态不正确");
    }

    String tradeNo = DateUtil.getDateNowYmd() + DateUtil.getRandom(14);

      boolean res = channelWithdrawService.update(new LambdaUpdateWrapper<ChannelWithdraw>()
          .set(ChannelWithdraw::getTradeNo, tradeNo)
          .set(ChannelWithdraw::getStatus, ChannelWithdrawStatusEnum.WITHDRAW_PROCESS)
          .eq(ChannelWithdraw::getId, channelWithdraw.getId())
          .eq(ChannelWithdraw::getStatus, ChannelWithdrawStatusEnum.WITHDRAWING));
      if (!res) {
          throw new ServiceException("提现状态更新失败");
      }


    OnlineReqDataVo req = new OnlineReqDataVo();
    req.setReqUrl(YsServerApiConstants.COMMON_API);
    req.setPrivateKeyFilePath(privateKeyPath);
    req.setYsPublicKeyFilePath(publicKeyPath);
    req.setPrivateKeyPassword(privateKeyPass);
    req.setNotifyUrl(ysWithdrawNotifyUrl);
    req.setPartnerId(partnerId);

    //实时提现（一般户到银行卡）业务参数
    Map<String, Object> bizContent = new HashMap<>();
    //商户生成的订单号，生成规则前8位必须为交易日期，如20180525，范围跨度支持包含当天在内的前后一天
    bizContent.put("out_trade_no", tradeNo);
    //商户号
    bizContent.put("merchant_usercode", partnerId);
    //暂时只支持币种：CNY（人民币）
    bizContent.put("currency", "CNY");
    //提现的总金额。单位为：RMB Yuan。取值范围为[0.01，99999999.99]，精确到小数点后两位。Number(10,2)指10位长度，2位精度
    bizContent.put("total_amount", new BigDecimal(channelWithdraw.getMoney()).divide(new BigDecimal(100), 2, RoundingMode.HALF_UP).toString());
    //订单说明
    bizContent.put("subject", "提现备注");
    //商户日期(该参数做交易与查询时需要一致) 该日期需在当日的前后一天时间范围之内
    bizContent.put("shopdate", DateUtil.getDateNowYmd());
    //银行帐号
    bizContent.put("bank_account_no", channelWithdraw.getCardNo());
    req.setParamData(bizContent);
    String result = null;
    try{
      logger.info("实时提现（一般户到银行卡）调用sdk接口addScanMerc请求入参为:"+ JSONObject.toJSONString(req));
      //根据返回结果处理自己的业务逻辑,result内容详见接口文档
      result = MercFundApi.withdrawQuick(req);
    }catch (Exception e){
      logger.info("实时提现（一般户到银行卡）失败:"+ e.getMessage());
    }

    logger.info("提现提交result = " + result);

    // 保存调用日志
      YsCallLog ysCallLog = new YsCallLog();
      ysCallLog.setInterfaceId(YsPayMethodConstants.SETTLED_ACCOUNT_WITHDRAW_METHOD);
      ysCallLog.setBizId(tradeNo);
      ysCallLog.setBizType(YsCallBizTypeEnum.WITHDRAW);
      ysCallLog.setReqJson(JSONObject.toJSONString(bizContent));
      ysCallLog.setResJson(result);
      ysCallLogService.save(ysCallLog);

      if (StringUtils.isBlank(result)) {
            throw new ServiceException("提现交易处理失败");
    }

    JSONObject response = JSONObject.parseObject(result);
    String tradeStatus = (String) response.get("trade_status");
    if (StringUtils.isBlank(tradeStatus) &&
        (!"TRADE_ACCEPT_SUCCESS".equals(tradeStatus) || !"TRADE_SUCCESS".equals(tradeStatus))) {
      logger.error("提现交易处理失败,result = " + result);
        throw new ServiceException("提现交易处理失败");
    }*/
    return true;
  }

  @Override
  public void payOrderStatusHandle(PayOrder payOrder) {
    String orderNo = payOrder.getOrderId();

    //查询统一支付订单表，查询订单类型，然后根据不同类型做对应的逻辑处理
    //查询支付状态为1：支付成功，2：订单创建成功，等待支付。
    LogUtil.info(logger, "支付回调,bizType:{0},bizId:{1}" ,
        new Object[]{payOrder.getBizType(), payOrder.getBizId()});
    if (BizTypeEnum.CHANNEL_ORDER == payOrder.getBizType()) {
      // 渠道盲票采购订单支付成功，调用业务订单服务处理
      channelOrderService.paySuccess(payOrder);
    } else if (BizTypeEnum.CHANNEL_GOODS_ORDER == payOrder.getBizType()) {
      // 门店商品采购订单支付成功，调用业务订单服务处理
      channelGoodsOrderService.paySuccess(payOrder);
    } else if (BizTypeEnum.TICKET_ORDER == payOrder.getBizType()) {
      // 用户盲票购买订单支付成功，调用业务订单服务处理
      userTicketOrderService.paySuccess(payOrder);
    } else if (BizTypeEnum.DELIVER_ORDER == payOrder.getBizType()) {
      // 用户提货订单支付成功，调用业务订单服务处理
      userDeliverOrderService.paySuccess(payOrder);
    } else {
      //数据结构异常，告警
      LogUtil.error(logger, "该业务单据类型未支持，orderNo：" + orderNo);
      throw new ServiceException("该业务单据类型未支持，orderNo：" + orderNo);
    }
  }


  @Override
  public boolean drawings(String merchId, int money) {
    return false;
  }

  @Override
  public boolean drawingsApply(String merchId, int money) {
    return false;
  }

  @Override
  public boolean completeDrawingsApply(String merchId) {
    return false;
  }


  private PayOrder buildPayOrder(JSONObject jsonObject) {
    PayOrder payOrder = jsonObject.toJavaObject(PayOrder.class);
    return payOrder;
  }

  private JSONObject buildPayOrderReqData(String shopNo, String sign, int money, String openId, String appId, String orderRemark,String payResource, String payResourceType) {

    JSONObject sendData = new JSONObject();
    sendData.put("shop_no" , shopNo);
    sendData.put("shop_order_no" , String.valueOf(bizIdGenerator.newId()));
    sendData.put("shop_order_time" , DateUtils.getTime());
    sendData.put("pay_resource" , payResource); //支付来源 1:支付宝  2:微信支付 5：云闪付
    sendData.put("trans_type_code" , payResourceType); //1017：微信小程序,1014支付宝
    sendData.put("transaction_amount" , money);
    sendData.put("order_remark" , "");
    sendData.put("timestamp" , System.currentTimeMillis());
    sendData.put("callback_url" , callbackUrl);
    // 截取备注长度
    sendData.put("orderName" , StringUtils.isNotBlank(orderRemark) && orderRemark.length() > 10 ? orderRemark.substring(0, 10) : orderRemark);
    sendData.put("sign" , sign);
    sendData.put("client_ip" , "113.31.163.91");

    sendData.put("auth_code" , openId);
    sendData.put("notify_url" , notifyUrl);
    // 微信小程序必传appId
    if ("2".equals(payResource)) {
      sendData.put("app_id" , appId);
    }
    return sendData;
  }
}
